import React from "react";
import {
  LOADING,
  PRE_GAME,
  PRE_ROUND,
  IN_ROUND,
  POST_ROUND,
  POST_GAME,
  ERROR,
} from "./game-state.enum";
import PreGameContainer from '../pre-game.component';
import PreRound from '../pre-round.component';
import GamePanelContainer from "../game-panel.component";
import RoundSummary from '../round-summary.component';
import PlayerScoresContainer from "../player-scores.component";
import Loading from "../loading.component";

class Game extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      googleApiKey: null,
      gameState: PRE_GAME,
      playerName: null,
      gameId: null,
      lastRound: null,
    }
  }
  
  renderLoading() {
    return <Loading/>
  }
  
  renderPreGame() {
    return <PreGameContainer 
      onGameJoined={({ gameId, playerName }) => this.setState({ gameState: PRE_ROUND, gameId, playerName })}
    />
  }
  
  renderPreRound() {
    const { gameId, playerName } = this.state;
    return <PreRound
    gameId={gameId}
    playerName={playerName}
    onStart={() => this.setState({ gameState: IN_ROUND })}
    />
  }
  
  renderInRound() {
    const { gameId, playerName } = this.state;
    return <GamePanelContainer
      gameId={gameId}
      playerName={playerName}
      onRoundEnd={lastRound => this.setState({ gameState: POST_ROUND, lastRound })}
      onGameEnd={() => this.setState({ gameState: POST_GAME })}
    />
  }
  
  renderPostRound() {
    const { lastRound } = this.state;
    const {
      roundNum,
      selectedPoint,
      targetPoint,
      score,
      totalScore,
    } = lastRound;
    return <RoundSummary
      roundNum={roundNum}
      score={score}
      totalScore={totalScore}
      onAdvanceState={() => this.setState({ gameState: IN_ROUND })}
      buttonText={roundNum === "5" ? "View Summary" : "Next Round"}
      targetPoint={targetPoint}
      selectedPoint={selectedPoint}
    />
  }
  
  renderPostGame() {
    const { gameId } = this.state;
    return <PlayerScoresContainer
      gameId={gameId}
      onReturnToStart={() => this.setState({ gameState: PRE_GAME })}
    />
  }
  
  renderError() {
    return <p>Application encountered unrecoverable error, please refresh the page.</p>
  }
  
  render() {
    const { gameState } = this.state;
    
    switch (gameState) {
      case LOADING:
      return this.renderLoading();
      case PRE_GAME:
      return this.renderPreGame();
      case PRE_ROUND:
      return this.renderPreRound();
      case IN_ROUND:
      return this.renderInRound();
      case POST_ROUND:
      return this.renderPostRound();
      case POST_GAME:
      return this.renderPostGame();
      case ERROR:
      return this.renderError();
      default:
      this.setState({ gameState: ERROR });
      return <p>Application state is inconsistent, please refresh and rejoin your previous game.</p>
    }
  }
}

export default Game;
